/**
 * The base class for several types of Animals.  Does not implement
 * the noise method of the Animal interface, hence must be abstract.
 */
public abstract class BasicAnimal implements Animal {
   /**
    * Create a new BasicAnimal, using supplied parameter values.
    * @param theName name of this animal
    * @param serving the size of a single meal for this animal in pounds
    * @param weight the initial weight of this animal in pounds
    */
    public BasicAnimal(String theName,double serving,double weight) {
        name = theName;
        mealSize = serving;
        currentWeight = weight;
        System.out.println("Created "+name);
    }
    /**
    * Provide this animal with a way to rest when weary.
    */
    public void sleep() {
        System.out.println(name+" : Snrf ... mutter ... snrf ...");
    }
    /** 
    * Eat some goodies.  There is some weight gain after eating.
    * @param pounds the number of pounds of food provided.
    */
    public void eat(double pounds) {
		double actual = Math.min(pounds,mealSize);
        System.out.print(name+" : Ate "+actual+" pounds of "+pounds+ " provided.");
        currentWeight = currentWeight + (actual/2);
        System.out.println("  Weight is now "+currentWeight+" pounds.");
    }
    /**
     * get the meal size defined for this animal.
     * @return meal size in pounds
     */
    public double getMealSize() {
        return mealSize;
    }
	/**
	 * print information about this animal.
	 * @return the string description of this animal
	 */
	 public String toString() {
		 return this.getClass().getName()+" "+name;
	 }
    
    //--------------------------------------------------------------------
    
    /** 
     * the serving size of one meal for this animal in pounds.
     */
    protected double mealSize;
    
    /** the weight of this animal in pounds. */
    protected double currentWeight;
    
    /** the name of this animal. */
    protected String name;
}
