import java.util.ArrayList;
/**
 * This class is a collection of animals of various types.  It
 * manages the collection by having all the various animals do various
 * things upon request.
 */
public class PetList {
    /**
     * This default constructor creates the new animal objects, using the
     * "new" operator and the appropriate constructors.  References to the
     * new animal objects are stored in instance variables of the 
     * PetList object.
	 * @param food total amount of food available in pounds
     */
    public PetList(double food) {
		remainingFood = food;
        theBunch = new ArrayList();
        theBunch.add(new Dog("Fred",1,20));
        theBunch.add(new Cat("Smoky",0.5,5));
        theBunch.add(new Sparrow("Libby",0.05,.1));
    }
    /**
     * Everybody in the set eats a small meal.  This method calls the
     * eat(food) method of each of the animal objects.  The amount of food
     * provided to each animal is the same as the defined meal size
     * for that animal, or the remaining amount of food, which ever
	 * is less.
     */
    public void snack() {
        for (int i=0; i<theBunch.size(); i++) {
			if (remainingFood <= 0) {
				System.out.println("No food left, so no more snacks.");
				break;
			}
            Animal pet = (Animal)theBunch.get(i);
            double s = Math.min(remainingFood,pet.getMealSize());
            pet.eat(s);
			remainingFood -= s;
        }
    }
    /**
     * Everybody in the set eats a large meal.  This method calls the 
     * eat(food) method of each of the animal objects.  The amount of food
     * provided to each animal is twice the amount of the defined
     * meal size for that animal.  After the meal, we throw away all
	 * the food that was provided to the animal, regardless of whether
	 * or not it was eaten.
     */
    public void dine() {
        for (int i=0; i<theBunch.size(); i++) {
            Animal pet = (Animal)theBunch.get(i);
            double s = 2*pet.getMealSize();
			if (remainingFood < s) {
				System.out.println("Not enough food for "+pet+"'s dinner, so we'll skip to next animal.");
				continue;
			}
            pet.eat(s);
			remainingFood -= s;
        }
    }
    /**
     * Everybody in the set sleeps it all off.  This method calls the
     * sleep() method of each of the animal objects.
     */
    public void sleep() {
        for (int i=0; i<theBunch.size(); i++) {
            ((Animal)theBunch.get(i)).sleep();
        }
    }
    /**
     * This method calls the noise-making method for each animal object.
     */
    public void speak() {
        for (int i=0; i<theBunch.size(); i++) {
            ((Animal)theBunch.get(i)).noise();
        }
    }
    /**
     * This is a simple test harness for this class.
     * @param args ignored
     */
    public static void main(String[] args) {
        PetList menagerie = new PetList(1);
        menagerie.speak();
        menagerie.snack();
        menagerie.dine();
        menagerie.sleep();
    }
    
    //------------------------------------------------------
    
    /** an ArrayList containing all the Animals */
    ArrayList theBunch;
	
	/** amount of food we still have left */
	double remainingFood;
}
